//LabVIEW-CIN implementation of IOWarrior 
//plus MAX6626 and MAX1238 on IIC bus
//R.X. 11-03, preliminary, reads only 3 channels
//For Mac OS X 10.3

#include <CoreFoundation/CoreFoundation.h>
#include "IOWarriorLib.h"
#include "extcode.h"

/* Typedefs */

typedef struct {
	int32 dimSize;
	uInt32 elt[1];
	} TD1;
typedef TD1 **TD1Hdl;


MgErr enableIICMode ();
MgErr readtemp(float32 *temperature);
MgErr write2pin (int);
MgErr CINRun(int16 *mode, float32 *Temp, int16 *pincode, TD1Hdl Volts);
MgErr readADC(unsigned int volts[]);

//====================================================


CIN MgErr CINLoad(RsrcFile reserved) {
    char	inBuffer[7];  
     int 	error;          
     
//initalize IOWarrior Lib
      IOWarriorInit ();

    if (!IOWarriorIsPresent ())
    {
        printf ("IOWarrior not connected\n");
        return -1;
    }

//reset pinouts
    write2pin (0x80);
    write2pin (0x00);
    
//enable IIC mode
      enableIICMode ();
      
//write pointer byte for temperature register
    bzero (inBuffer, 7);
    inBuffer[0] = 0x82; //flags for start plus announce 2 bytes: 1000 0010
    inBuffer[1] = 0x92; //address plus write bit: 1001001 0
    inBuffer[2] = 0x00; //pointer byte for temperature register 
    error = IOWarriorWriteInterface1 (2, inBuffer);
    if (error)
    {
        return -1;
    }        
        return 0;      
}

//====================================================

// for MAX6626
MgErr readtemp(float32 *temperature)
{
    int 	error, temp;         
    char	inBuffer[7];  
    
//setup report buffer
    bzero (inBuffer, 7);
    inBuffer[0] = 0x02; //read two bytes of data
    inBuffer[1] = 0x93; //address plus read bit: 1001001 1
    
//write report
    error = IOWarriorWriteInterface1 (3, inBuffer);
           if (error)
    {
        return -1;
    }
   
//read report
    error = IOWarriorReadInterface1 (3, inBuffer);
           if (error)
    {
        return -1;
    }
     
    temp =
        16*16*(inBuffer[2]>>4)+16*(inBuffer[2]-(16*(inBuffer[2]>>4)))+(inBuffer[3]>>4);      
    *temperature = .0625 * temp;
    
    return 0;
}

//====================================================

//write to LED pins at port 3
MgErr write2pin (int pincode)     
{
    unsigned char buffer[4];
    int error; 
    
    bzero (buffer, 4);    
    buffer[3]=~pincode;
    
//write 32 bits   
    error = IOWarriorWriteInterface0 (buffer);
    if (error)
    {
        return -1;
    }
    return 0;
}


//====================================================

//for MAX1238EEE, reads channels 0,1,2
MgErr readADC(unsigned int volts[])
{
    int 	error;         //int has 4 bits
    char	inBuffer[7];   //char has 1 byte
    unsigned char	outBuffer[8];   
    IOWarriorListNode* theNode;

//write setup and config bytes
    bzero (inBuffer, 7);
    inBuffer[0] = 0xc3; //flags for start/stop plus announce 3 bytes: 1100 0011
    inBuffer[1] = 0x6a; //address plus write bit: 0110 101 0  for model EEE

//now 2 bytes:    
    inBuffer[2] = 0xfa; // 1(setup) 111(ref sel) - 1(ext clk) 0(unip) 1(no action)0(nc) ok        
    inBuffer[3] = 0x05;  // 0(config) 00(scanning) 0 - 010(ainX)    1 (sgl)    we scan  ain0-ain2
             
//write:        
   error = IOWarriorWriteInterface1 (2, inBuffer);
       if (error)
    {
        return -1;
    }
        
//start read cycle by sending request
    bzero (inBuffer, 7);
    inBuffer[0] = 0x06; //read two bytes of data  (need four if we read 2 channels etc)
    inBuffer[1] = 0x6b; //address plus read bit: 0110 101 1 for model EEE

    error = IOWarriorWriteInterface1 (3, inBuffer);
          if (error)
    {
        return -1;
    }

     bzero (outBuffer, 8);
//really read
//    error = IOWarriorReadInterface1 (3, &outBuffer);
    theNode = IOWarriorInterfaceListNodeAtIndex(1); 
error = IOWarriorReadFromInterface(theNode->ioWarriorHIDInterface,3,8,&outBuffer);
           if (error)
    {
        return -1;
    }

    volts[0]=16*16*(outBuffer[2]-16*(outBuffer[2]>>4))+outBuffer[3];
    volts[1]=16*16*(outBuffer[4]-16*(outBuffer[4]>>4))+outBuffer[5];
    volts[2]=16*16*(outBuffer[6]-16*(outBuffer[6]>>4))+outBuffer[7];

   return 0;
}


//====================================================

MgErr enableIICMode ()
{
    char buffer[7];
    int	error;

    bzero (buffer, 7);
    buffer[0] = 1; // set enable bit
    error = IOWarriorWriteInterface1 (1, buffer);
               if (error)
    {
	return -1;
    }
    
    return 0;
}

//====================================================

//crashes
/* Make this CIN reentrant. 
CIN MgErr CINProperties(int32 query, void *data)
    {
    switch (query) {
        case kCINIsReentrant:           // are you reentrant? 
            *(Bool32 *)data = TRUE;         // yes
            return noErr;               
        }
    return mgNotSupported;          
    }
*/

//====================================================


MgErr CINRun(int16 *mode, float32 *temp, int16 *pincode, TD1Hdl Volts)
 {
 int error;
// int32 ADCOutputChannels;
 int noErr=0;
 float32 Temp=0.;
 uInt32 *volts;
 unsigned int voltsout[3];
 
 
  
if(*mode == 1)
    {
    error = readtemp(&Temp);
    *temp=Temp;
    return error;
    }

if(*mode == 2)
    {
    error= write2pin (*pincode);
    return error;
    }

if(*mode ==3)
    {
// ADCOutputChannels=(*Volts)->dimSize;
 volts=(*Volts)->elt;
error = readADC(voltsout);
volts[0]=voltsout[0];
volts[1]=voltsout[1];
volts[2]=voltsout[2];
//    error=0;
    return error;
    }

    return noErr;
}
        
        
        